@@ -292,7 +292,7 @@ cpu_profiling_setupstart(perf_cpu_t *cpu,
 static int
 cpu_profiling_partpause(perf_cpu_t *cpu, void *arg)
 {
-	perf_count_id_t perf_count_id = (perf_count_id_t)arg;
+  	perf_count_id_t perf_count_id = (perf_count_id_t)(uintptr_t)arg;
 	int i;
 
 	if (perf_count_id == PERF_COUNT_INVALID ||
@@ -345,7 +345,7 @@ cpu_profiling_multipause(perf_cpu_t *cpu, void *arg)
 static int
 cpu_profiling_restore(perf_cpu_t *cpu, void *arg)
 {
-	perf_count_id_t perf_count_id = (perf_count_id_t)arg;
+  	perf_count_id_t perf_count_id = (perf_count_id_t)(uintptr_t)arg;
 	int i;
 
 	if (perf_count_id == PERF_COUNT_INVALID ||
@@ -1510,7 +1510,7 @@ os_uncore_start(perf_ctl_t *ctl, perf_task_t *task)
 
 	if (uncore_start(ctl, t->nid) != 0) {
 		debug_print(NULL, 2,
-			"os_uncore_start is failed for node %d/%d\n",
+			"os_uncore_start failed for node %d\n",
 			t->nid);
 		perf_status_set(PERF_STATUS_UNCORE_FAILED);
 		return (-1);
