@@ -27,6 +27,7 @@
  */
 
 #define _GNU_SOURCE
+#define _XOPEN_SOURCE 500
 #include <inttypes.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -41,7 +42,9 @@
 #include <limits.h>
 #include <locale.h>
 #include <math.h>
+#include <ftw.h>
 #include <sys/wait.h>
+#include <sys/mount.h>
 #include "../include/types.h"
 #include "../include/util.h"
 #include "../include/os/os_util.h"
@@ -387,12 +390,10 @@ str_int_extract(char *str, int *arr, int arr_size, int *num)
 	int len = strlen(str);
 	boolean_t ret = B_FALSE;
 
-	if ((scopy = malloc(len + 1)) == NULL) {
+	if ((scopy = strdup(str)) == NULL) {
 		return (B_FALSE);
 	}
 
-	strncpy(scopy, str, len);
-	scopy[len] = 0;
 	cur = scopy;
 
 	while (cur < (scopy + len)) {
@@ -447,7 +448,6 @@ file_int_extract(char *path, int *arr, int arr_size, int *num)
 		fclose(fp);
 		return (B_FALSE);
 	}
-
 	fclose(fp);
 	return (str_int_extract(buf, arr, arr_size, num));
 }
@@ -701,23 +701,6 @@ os_sysfs_uncore_imc_init(imc_info_t *imc, int num)
 	return imc_num;
 }
 
-static boolean_t execute_command(const char *command, const char *type)
-{
-	FILE *fp;
-
-	fp = popen(command, type);
-	if (fp == NULL) {
-		debug_print(NULL, 2, "Execute '%s' failed (errno = %d)\n",
-			command, errno);
-		return B_FALSE;
-	}
-
-	pclose(fp);
-	debug_print(NULL, 2, "Execute '%s' ok\n", command);
-
-	return B_TRUE;
-}
-
 static boolean_t resctrl_mounted(void)
 {
 	char path[128];
@@ -734,39 +717,72 @@ static boolean_t resctrl_mounted(void)
 
 boolean_t os_cmt_init(void)
 {
-	char command[128];
+	int ret;
 
 	g_pqos_moni_id = 0;
 
 	if (resctrl_mounted())
 		return B_TRUE;
 
-	snprintf(command, sizeof(command),
-		"mount -t resctrl resctrl /sys/fs/resctrl 2>/dev/null");
-
-	if (!execute_command(command, "r"))
+	ret = mount("resctrl", "/sys/fs/resctrl", "resctrl", 0, NULL);
+	if (ret < 0) {
+		debug_print(NULL, 2, "Mount of /sys/fs/resctrl failed (errno = %d)\n", errno);
 		return B_FALSE;
+	}
 
 	return resctrl_mounted();
 }
 
 void os_cmt_fini(void)
 {
-	char command[128];
+	int ret;
 
 	if (!resctrl_mounted())
 		return;
 
-	snprintf(command, sizeof(command),
-		"umount -f /sys/fs/resctrl 2>/dev/null");
+	ret = umount("/sys/fs/resctrl");
+	if (ret < 0) {
+		debug_print(NULL, 2, "Unmount of /sys/fs/resctrl failed (errno = %d)\n", errno);
+	}
 
-	execute_command(command, "r");
 	g_pqos_moni_id = 0;
 }
 
+static int os_sysfs_nftw_unlink(const char *fpath, const struct stat *statbuf,
+			       int tflag, struct FTW *ftwbuf)
+{
+	int ret;
+
+	(void)statbuf;
+	(void)ftwbuf;
+
+	switch (tflag) {
+	case FTW_F:
+	case FTW_SL:
+	case FTW_SLN:
+	default:
+		errno = 0;
+		ret = unlink(fpath);
+		if (ret < 0)
+			debug_print(NULL, 2, "unlink %s (errno=%d)\n", fpath, errno);
+		break;
+	case FTW_D:
+	case FTW_DP:
+	case FTW_DNR:
+		errno = 0;
+		ret = rmdir(fpath);
+		if (ret < 0)
+			debug_print(NULL, 2, "rmdir %s (errno=%d)\n", fpath, errno);
+		break;
+	}
+
+	return 0;
+}
+
 int os_sysfs_cmt_task_set(int pid, int lwpid, struct _perf_pqos *pqos)
 {
-	char command[160], path[128];
+	char data[64], path[128];
+	int ret, fd;
 
 	if (lwpid)
 		pqos->task_id = lwpid;
@@ -778,23 +794,33 @@ int os_sysfs_cmt_task_set(int pid, int lwpid, struct _perf_pqos *pqos)
 	snprintf(path, sizeof(path),
 		"/sys/fs/resctrl/mon_groups/%d", pqos->task_id);
 
-	snprintf(command, sizeof(command), "rm -rf %s 2>/dev/null", path);
-	if (!execute_command(command, "r"))
-		return -1;
+	/* nftw will return -1 if the path does not exist, ignore this */
+	(void)nftw(path, os_sysfs_nftw_unlink, 32, FTW_DEPTH | FTW_PHYS);
 
-	snprintf(command, sizeof(command), "mkdir %s 2>/dev/null", path);
-	if (!execute_command(command, "r"))
+	ret = mkdir(path, 0777);
+	if (ret < 0) {
+		debug_print(NULL, 2, "mkdir %s failed, errno=%d\n", path, errno);
 		return -1;
+	}
+
+	snprintf(path, sizeof(path),
+		"/sys/fs/resctrl/mon_groups/%d/tasks", pqos->task_id);
+	fd = open(path, O_RDWR);
+	if (fd < 0) {
+		debug_print(NULL, 2, "open %s failed, errno=%d\n", path, errno);
+		return -1;
+	}
 
 	if (lwpid == 0)
-		snprintf(command, sizeof(command),
-			"echo %d > %s/tasks", pid, path);
+		snprintf(data, sizeof(data), "%d\n", pid);
 	else
-		snprintf(command, sizeof(command),
-			"echo %d > %s/tasks", lwpid, path);
-
-	if (!execute_command(command, "r"))
+		snprintf(data, sizeof(data), "%d\n", lwpid);
+	if (write(fd, data, strlen(data)) < 0) {
+		debug_print(NULL, 2, "write to %s failed, errno=%d\n", path, errno);
+		(void)close(fd);
 		return -1;
+	}
+	(void)close(fd);
 
 	return 0;
 }
