@@ -29,6 +29,7 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -105,6 +106,28 @@ cpuid_cmp(const void *a, const void *b)
 	return (0);
 }
 
+static void
+print_buf(char **destp, int *sizep, const char *fmt, ...)
+{
+	va_list ap;
+	int len;
+
+	if (*sizep <= 0)
+		return;
+
+	va_start(ap, fmt);
+	len = vsnprintf(*destp, *sizep, fmt, ap);
+	va_end(ap);
+
+	if (len >= *sizep) {
+		*sizep = 0;
+		return;
+	}
+
+	*destp += len;
+	*sizep -= len;
+}
+
 /*
  * Build a readable string of CPU ID and try to reduce the string length. e.g.
  * For cpu1, cpu2, cpu3, cpu4, the string is "CPU(1-4)",
@@ -113,7 +136,6 @@ cpuid_cmp(const void *a, const void *b)
 static void
 node_cpu_string(node_t *node, char *s1, int size)
 {
-	char s2[128], s3[128];
 	int i, j, k, l, cpuid_start;
 	int *cpuid_arr;
 	int ncpus;
@@ -140,8 +162,7 @@ node_cpu_string(node_t *node, char *s1, int size)
 	cpuid_start = cpuid_arr[0];
 
 	if (ncpus == 1) {
-		(void) snprintf(s2, sizeof (s2), "%d", cpuid_start);
-        	(void) strncat(s1, s2, strlen(s2));
+		(void) snprintf(s1, size, "%d", cpuid_start);
         	free(cpuid_arr);
 		return;
 	}
@@ -154,33 +175,27 @@ node_cpu_string(node_t *node, char *s1, int size)
 		if (cpuid_arr[j] != cpuid_start + l) {
 			if (k < ncpus) {
 				if (l == 1) {
-					(void) snprintf(s2, sizeof (s2), "%d ", cpuid_start);
+					print_buf(&s1, &size, "%d ", cpuid_start);
 				} else {
-					(void) snprintf(s2, sizeof (s2),
+					print_buf(&s1, &size,
 						"%d-%d ", cpuid_start, cpuid_start + l - 1);
 				}
           		} else {
 				if (l == 1) {
-					(void) snprintf(s2, sizeof (s2), "%d",
-						cpuid_start);
+					print_buf(&s1, &size, "%d", cpuid_start);
 				} else {
-					(void) snprintf(s2, sizeof (s2), "%d-%d",
+					print_buf(&s1, &size, "%d-%d",
 						cpuid_start, cpuid_start + l - 1);
 				}
-
-				(void) snprintf(s3, sizeof (s3), " %d",
-					cpuid_arr[j]);
-	        	  	(void) strncat(s2, s3, strlen(s3));
+				print_buf(&s1, &size, " %d", cpuid_arr[j]);
 			}
 
-          		(void) strncat(s1, s2, strlen(s2));
           		cpuid_start = cpuid_arr[j];
            		l = 1;
 		} else {
 	        	if (k == ncpus) {
-        	    		(void) snprintf(s2, sizeof (s2), "%d-%d",
+				print_buf(&s1, &size, "%d-%d",
                 			cpuid_start, cpuid_start + l);
-         			(void) strncat(s1, s2, strlen(s2));
        			} else {
             			l++;
        			}
@@ -206,7 +221,7 @@ nodedetail_line_show(win_reg_t *reg, char *title, char *value, int line)
 void
 os_nodedetail_data(dyn_nodedetail_t *dyn, win_reg_t *seg)
 {
-	char s1[256], s2[32];
+	char s1[512], s2[32];
 	node_t *node;
 	win_countvalue_t value;
 	node_meminfo_t meminfo;
@@ -561,7 +576,7 @@ os_llcallchain_win_destroy(dyn_win_t *win)
 static int
 bufaddr_cmp(const void *p1, const void *p2)
 {
-	const uint64_t addr = (const uint64_t)p1;
+	const uint64_t addr = (const uint64_t)(uintptr_t)p1;
 	const bufaddr_t *bufaddr = (const bufaddr_t *)p2;
 
 	if (addr < bufaddr->addr) {
@@ -621,7 +636,7 @@ llcallchain_bufinfo_show(dyn_llcallchain_t *dyn, track_proc_t *proc,
 	 * Check if the linear address is located in a buffer in
 	 * process address space.
 	 */ 
-	if ((line = bsearch((void *)(dyn->addr), lat_buf, nlines,
+	if ((line = bsearch((void *)(uintptr_t)(dyn->addr), lat_buf, nlines,
 		sizeof (lat_line_t), bufaddr_cmp)) != NULL) {
 		win_lat_str_build(content, WIN_LINECHAR_MAX, 0, line);
 		reg_line_write(reg, 0, ALIGN_LEFT, content);
@@ -791,7 +806,7 @@ os_lat_buf_hit(lat_line_t *lat_buf, int nlines, os_perf_llrec_t *rec,
 	 * Check if the linear address is located in a buffer in
 	 * process address space.
 	 */
-	if ((line = bsearch((void *)(rec->addr), lat_buf, nlines,
+	if ((line = bsearch((void *)(uintptr_t)(rec->addr), lat_buf, nlines,
 	    sizeof (lat_line_t), bufaddr_cmp)) != NULL) {
 		/*
 		 * If the linear address is located in, that means this
