@@ -58,7 +58,7 @@ mmap_buffer_read(struct perf_event_mmap_page *header, void *buf, size_t size)
 {
 	void *data;
 	uint64_t data_head, data_tail;
-	int data_size, ncopies;
+	int ncopies;
 	
 	/*
 	 * The first page is a meta-data page (struct perf_event_mmap_page),
@@ -79,7 +79,7 @@ mmap_buffer_read(struct perf_event_mmap_page *header, void *buf, size_t size)
 	 * The kernel function "perf_output_space()" guarantees no data_head can
 	 * wrap over the data_tail.
 	 */
-	if ((data_size = data_head - data_tail) < (int)size) {
+	if ((int64_t)(data_head - data_tail) < (int64_t)size) {
 		return (-1);
 	}
 
@@ -166,7 +166,7 @@ pf_profiling_setup(struct _perf_cpu *cpu, int idx, pf_conf_t *conf)
 		PERF_FORMAT_TOTAL_TIME_ENABLED | PERF_FORMAT_TOTAL_TIME_RUNNING;
 	attr.size = sizeof(attr);
 
-	debug_print(NULL, 2, "pf_profiling_setup: attr.type = 0x%lx, "
+	debug_print(NULL, 2, "pf_profiling_setup: attr.type = 0x%x, "
 		"attr.config = 0x%lx, attr.config1 = 0x%lx\n",
 		 attr.type, attr.config, attr.config1);
 
@@ -432,9 +432,10 @@ pf_ll_setup(struct _perf_cpu *cpu, pf_conf_t *conf)
 	attr.config1 = conf->config1;
 	attr.sample_period = conf->sample_period;
 	attr.precise_ip = 1;
-	attr.exclude_guest = 1;
+	attr.exclude_guest = conf->exclude_guest;
 	attr.sample_type = PERF_SAMPLE_TID | PERF_SAMPLE_ADDR | PERF_SAMPLE_CPU |
-		PERF_SAMPLE_WEIGHT | PERF_SAMPLE_CALLCHAIN;
+		PERF_SAMPLE_WEIGHT | PERF_SAMPLE_CALLCHAIN |
+		PERF_SAMPLE_DATA_SRC;
 	attr.disabled = 1;
 
 	if ((fds[0] = pf_event_open(&attr, -1, cpu->cpuid, -1, 0)) < 0) {
@@ -481,6 +482,7 @@ ll_sample_read(struct perf_event_mmap_page *mhdr, int size,
 	pf_ll_rec_t *rec)
 {
 	struct { uint32_t pid, tid; } id;
+	union perf_mem_data_src data_src;
 	uint64_t i, addr, cpu, weight, nr, value, *ips;
 	int j, ret = -1;
 
@@ -492,6 +494,7 @@ ll_sample_read(struct perf_event_mmap_page *mhdr, int size,
 	 *	[ u64	nr; }
 	 *	{ u64   ips[nr]; }
 	 *	{ u64	weight; }
+	 *	{ u64   data_src; }
 	 * };
 	 */
 	if (mmap_buffer_read(mhdr, &id, sizeof (id)) == -1) {
@@ -551,7 +554,18 @@ ll_sample_read(struct perf_event_mmap_page *mhdr, int size,
 	}
 
 	size -= sizeof (weight);
-	
+
+	if (mmap_buffer_read(mhdr, &data_src, sizeof (data_src)) == -1) {
+		debug_print(NULL, 2, "ll_sample_read: read data_src failed.\n");
+		goto L_EXIT;
+	}
+
+	size -= sizeof (data_src);
+
+	if (data_src.mem_op == PERF_MEM_OP_NA ||
+	    data_src.mem_op == PERF_MEM_OP_EXEC)
+		addr = 0;
+
 	rec->ip_num = j;
 	rec->pid = id.pid;
 	rec->tid = id.tid;
@@ -575,7 +589,7 @@ ll_recbuf_update(pf_ll_rec_t *rec_arr, int *nrec, pf_ll_rec_t *rec)
 {
 	int i;
 
-	if ((rec->pid == 0) || (rec->tid == 0)) {
+	if ((rec->pid == 0) || (rec->tid == 0) || (rec->addr == 0)) {
 		/* Just consider the user-land process/thread. */
 		return;	
 	}
@@ -759,14 +773,14 @@ pf_uncoreqpi_setup(struct _node *node)
 		if ((qpi->qpi_info[i].fd = pf_event_open(&attr, -1,
 			node->cpus[0].cpuid, -1, 0)) < 0) {
 			debug_print(NULL, 2, "pf_uncoreqpi_setup: pf_event_open is failed "
-				"for node %d, qpi %d, cpu %d, type %d, config 0x%x\n",
+				"for node %d, qpi %d, cpu %d, type %d, config 0x%lx\n",
 				node->nid, i, node->cpus[0].cpuid, attr.type, attr.config);
 			qpi->qpi_info[i].fd = INVALID_FD;
 			return (-1);
 		}
 
 		debug_print(NULL, 2, "pf_uncoreqpi_setup: pf_event_open is successful "
-			"for node %d, qpi %d, cpu %d, type %d, config 0x%x, fd %d\n",
+			"for node %d, qpi %d, cpu %d, type %d, config 0x%lx, fd %d\n",
 			node->nid, i, node->cpus[0].cpuid, attr.type, attr.config,
 			qpi->qpi_info[i].fd);
 	}
@@ -883,14 +897,14 @@ pf_uncoreimc_setup(struct _node *node)
 		if ((imc->imc_info[i].fd = pf_event_open(&attr, -1,
 			node->cpus[0].cpuid, -1, 0)) < 0) {
 			debug_print(NULL, 2, "pf_uncoreimc_setup: pf_event_open is failed "
-				"for node %d, imc %d, cpu %d, type %d, config 0x%x\n",
+				"for node %d, imc %d, cpu %d, type %d, config 0x%lx\n",
 				node->nid, i, node->cpus[0].cpuid, attr.type, attr.config);
 			imc->imc_info[i].fd = INVALID_FD;
 			return (-1);
 		}
 
 		debug_print(NULL, 2, "pf_uncoreimc_setup: pf_event_open is successful "
-			"for node %d, imc %d, cpu %d, type %d, config 0x%x, fd %d\n",
+			"for node %d, imc %d, cpu %d, type %d, config 0x%lx, fd %d\n",
 			node->nid, i, node->cpus[0].cpuid, attr.type, attr.config,
 			imc->imc_info[i].fd);
 	}
