@@ -384,7 +384,7 @@ elf32_binary_read(sym_binary_t *binary, sym_type_t sym_type)
 	}
 
 	if (ehdr.e_shentsize != sizeof (Elf32_Shdr)) {
-		debug_print(NULL, 2, "elf32_binary_read: ehdr.e_shentsize != %d\n",
+		debug_print(NULL, 2, "elf32_binary_read: ehdr.e_shentsize != %zu\n",
 			sizeof (Elf32_Shdr));
 		return (-1);
 	}
@@ -463,7 +463,7 @@ L_EXIT:
 }
 
 static int
-elf64_binary_read(sym_binary_t *binary, unsigned int sym_type)
+elf64_binary_read(sym_binary_t *binary, sym_type_t sym_type)
 {
 	Elf64_Ehdr ehdr;
 	Elf64_Shdr shdr;
@@ -481,7 +481,7 @@ elf64_binary_read(sym_binary_t *binary, unsigned int sym_type)
 	}
 
 	if (ehdr.e_shentsize != sizeof (Elf64_Shdr)) {
-		debug_print(NULL, 2, "elf64_binary_read: ehdr.e_shentsize != %d\n",
+		debug_print(NULL, 2, "elf64_binary_read: ehdr.e_shentsize != %zu\n",
 			sizeof (Elf64_Shdr));
 		return (-1);
 	}
@@ -842,7 +842,7 @@ sym_resolve(sym_t *sym, uint64_t addr, sym_item_t **item_arr,
 static sym_item_t *
 resolve_unique(sym_t *sym, uint64_t addr, sym_item_t **arr, uint64_t *base_addr)
 {
-	sym_item_t *item_arr, *item;
+	sym_item_t *item_arr, *item = NULL;
 	int num, i;
 
 	if (sym_resolve(sym, addr, &item_arr, &num, base_addr) != 0) {
